<?php

namespace WebsiteBundle\Entity;

use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Context\ExecutionContextInterface;

/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class PsychEducationResponse extends BaseEntity {

  /**
   * @ORM\ManyToOne(targetEntity="PsychEducationPanel", inversedBy="responses")
   * @var PsychEducationPanel
   */
  private $panel;

  /**
   * @ORM\Column(type="string")
   * @var string
   * @Assert\NotBlank
   */
  private $value;

  /**
   * @ORM\Column(type="boolean")
   * @var bool
   */
  private $isCorrect = false;

  /**
   * @ORM\Column(type="text", nullable=true)
   * @var string
   */
  private $incorrectReason;

  /**
   * @return PsychEducationPanel
   */
  public function getPanel()
  {
    return $this->panel;
  }

  /**
   * @param PsychEducationPanel $panel
   * @return $this
   */
  public function setPanel(PsychEducationPanel $panel)
  {
    $this->panel = $panel;

    return $this;
  }

  /**
   * @return string
   */
  public function getValue()
  {
    return $this->value;
  }

  /**
   * @param string $value
   * @return $this
   */
  public function setValue($value)
  {
    $this->value = $value;

    return $this;
  }

  /**
   * @return bool
   */
  public function isCorrect()
  {
    return $this->isCorrect;
  }

  /**
   * @param bool $isCorrect
   * @return $this
   */
  public function setIsCorrect($isCorrect)
  {
    $this->isCorrect = $isCorrect;

    return $this;
  }

  /**
   * @return string
   */
  public function getIncorrectReason()
  {
    return $this->incorrectReason;
  }

  /**
   * @param string $incorrectReason
   * @return $this
   */
  public function setIncorrectReason($incorrectReason)
  {
    $this->incorrectReason = $incorrectReason;

    return $this;
  }

  /**
   * @Assert\Callback
   * @param ExecutionContextInterface $context
   * @param $payload
   */
  public function validate(ExecutionContextInterface $context, $payload)
  {
    if (!$this->isCorrect) {
      //Check if a question was added
      if (empty($this->incorrectReason)) {
        $context->buildViolation('admin.entities.psychEducationResponse.needIncorrectReason')
          ->atPath('incorrectReason')
          ->addViolation();
      }
    }
  }
}
