<?php
// Thanks to https://gist.github.com/bjo3rnf/4061232
namespace Bazookas\AdminBundle\Form\DataTransformer;

use Symfony\Component\Form\DataTransformerInterface;
use Symfony\Component\Form\Exception\TransformationFailedException;
use Doctrine\Common\Persistence\ObjectManager;

class EntityToIDTransformer implements DataTransformerInterface
{
  /**
   * @var ObjectManager
   */
  protected $objectManager;
  /**
   * @var string
   */
  protected $class;

  public function __construct(ObjectManager $objectManager, $options)
  {
    $this->objectManager = $objectManager;
    $this->class = $options['class'];
  }
  public function transform($entity)
  {
    if (null === $entity) {
      return;
    }

    $meta = $this->objectManager->getClassMetadata($this->class);
    $identifier = $meta->getIdentifier()[0];
    if ($identifier === 'id') {
      return $entity->getId();
    }
    else {
      $methodName = 'get'.ucfirst($identifier);
      if (method_exists($entity, $methodName)) {
        return $entity->{$methodName}();
      }
    }
    throw new TransformationFailedException();
  }
  public function reverseTransform($id)
  {
    if (!$id) {
      return null;
    }

    $entity = $this
      ->objectManager
      ->getRepository($this->class)
      ->find($id);

    if (null === $entity) {
      throw new TransformationFailedException();
    }

    return $entity;
  }

  public function setClass($class) {
    $this->class = $class;
    return $this;
  }
}
