## Versioning entities

This component allows you to version your entities.

### Getting Started
#### Configuration

You must enable the versioning component in `config.yml` by settings the `enableVersioning` key to true:

```yaml
bazookas_admin:
  audit:
    enableVersioning: true
```

#### Basic Usage with entities

Let's assume you have the following entity:

```php
<?php

namespace WebsiteBundle\Entity;

use Bazookas\CommonBundle\Entity\Base\BaseEntity;

/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class Model extends BaseEntity {

}
```

Enable versioning for the entity by adding the `Versionable` interface:

```php
<?php

namespace WebsiteBundle\Entity;

use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Bazookas\AdminBundle\Entity\Interfaces\Versionable;
use Bazookas\AdminBundle\Entity\Traits\VersionableTrait;

/**
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class Model extends BaseEntity implements Versionable {

  use VersionableTrait;

  /**
   * @ORM\OneToMany(targetEntity="Model", mappedBy="parent")
   */
  protected $children;

  /**
   * @ORM\ManyToOne(targetEntity="Model", inversedBy="children")
   */
  protected $parent;
}
```

The `VersionableTrait` shown above provides a default implementation of the versionable interface. Please note that it is always necessary to separately define the `children` and `parent` properties on your entity even if you implement the `VersionableTrait`. The `children` and `parent` properties are self-referencing properties and should always point to the current entity.

#### Using mutable versioning

//TODO
