<?php

namespace Bazookas\APIFrameworkBundle\Formatter\Base;
use Bazookas\APIFrameworkBundle\Exception\InvalidArgumentException;

/**
 * @deprecated use OutputFormatter instead
 */
class BaseFormatter
{
  //conversion types
  const TYPE_STRING = 'string';
  const TYPE_SCALAR_ARRAY = 'array_scalar';
  const TYPE_SCALAR = 'scalar';
  const TYPE_INTEGER = 'integer';
  const TYPE_ENTITY_ARRAY = 'entity_array';

  protected $config;
  protected $globalConfig;

  public function __construct($config, $globalConfig = []) {
    $this->config = $config;
    $this->globalConfig = $globalConfig;
  }

  public function format(&$data) {
    if ((empty($this->config) && empty($this->globalConfig)) || empty($data)) {
      return $data;
    }

    if (is_object($data)) {

      //TODO implement this

      return $this->formatObject();
    } else {
      if (!empty($this->config)) {
        //apply configurations
        foreach($this->config as $key => $transformation) {
          if (isset($data[0]) && is_array($data[0])) {
            foreach($data as &$element) {
              if (isset($element[$key])) {
                $this->formatArrayObject($key, $element, $transformation);
              }
            }
          } else {
            if (isset($data)) {
              if (is_object($data)) {
                $this->formatObject($key, $data, $transformation);
              } else {
                $this->formatArrayObject($key, $data, $transformation);
              }
            }
          }
        }
      }

      //apply the global transformations
      if (!empty($this->globalConfig)) {
        foreach($this->globalConfig as $transformation) {
          if (isset($transformation['conversionFormat'])) {
            $conversionParams = [];
            if (isset($transformation['conversionParams'])) {
              $conversionParams = $transformation['conversionParams'];
            }

            $data = $this->executeConversion($transformation['conversionFormat'], $data, $conversionParams);
          }
        }
      }

      return $data;
    }
  }

  protected function formatObject($fieldKey, $element, $transformation) {

    //TODO implement this
    throw new \Exception('Not implemented');

  }

  protected function formatArrayObject($fieldKey, array &$data, $transformation) {
    //Convert the value to a specified format
    if (isset($transformation['conversionFormat'])) {
      $conversionParams = [];
      if (isset($transformation['conversionParams'])) {
        $conversionParams = $transformation['conversionParams'];
      }

      $format = $transformation['conversionFormat'];
      if (isset($data[$fieldKey])) {
        $data[$fieldKey] = $this->convertToType($transformation['convertToType'], $format, $data[$fieldKey], $conversionParams);
      }
    }

    //Apply fieldKey transformations if necessary
    if (isset($transformation['fieldKey'])) {
      $fieldKeyTransformation = $transformation['fieldKey'];
      $oldKey = $fieldKey;

      if (isset($fieldKeyTransformation['newKey'])) {
        $fieldKey = $fieldKeyTransformation['newKey'];
      }

      if (isset($fieldKeyTransformation['conversionFormat'])) {
        $conversionParams = [];
        if (isset($fieldKeyTransformation['conversionParams'])) {
          $conversionParams = $fieldKeyTransformation['conversionParams'];
        }

        $type = isset($fieldKeyTransformation['convertToType']) ? $fieldKeyTransformation['convertToType'] : self::TYPE_STRING;
        $fieldKey = $this->convertToType($type, $fieldKeyTransformation['conversionFormat'], $fieldKey, $conversionParams);
      }

      //apply the key change
      if ($oldKey != $fieldKey) {
        $data[$fieldKey] = $data[$oldKey];
        unset($data[$oldKey]);
      }
    }
  }

  protected function convertToType($type, $format, $value, $conversionParams = []) {
    switch ($type) {
      case self::TYPE_STRING:
        if (empty($format)) {
          return (string) $value;
        } else {
          return (string) $this->executeConversion($format, $value, $conversionParams);
        }
        break;
      case self::TYPE_SCALAR_ARRAY:
      case self::TYPE_SCALAR:
        if (!is_callable($format) && !is_array($format)) {
          throw new InvalidArgumentException('Unknown formatting');
        } else {
          return $this->executeConversion($format, $value, $conversionParams);
        }
        break;
      case self::TYPE_INTEGER:
        if (empty($format)) {
          return (int) $value;
        } else {
          return (int) $this->executeConversion($format, $value, $conversionParams);
        }
        break;
      case self::TYPE_ENTITY_ARRAY:
        if ($format instanceof BaseFormatter){
          return $format->format($value);
        } else {
          throw new \Exception('Unknown format \''.$format.'\' of type\''.$type.'\' with value \''.$value.'\' expected object of type Bazookas\APIFrameworkBundle\Formatter\Base\BaseFormatter');
        }
      default:
        return $value;
    }
  }

  protected function executeConversion($format, $value, $conversionParams) {
    if (is_callable($format)) {
      if (!empty($conversionParams)) {
        array_unshift($conversionParams, $value);
        return call_user_func_array($format, $conversionParams);
      } else {
        return call_user_func($format, $value);
      }
    } else {
      return $value;
    }
  }

  /* ---------------------------------------------------------------------------------------------
   *                              Default field transformation functions
   * ---------------------------------------------------------------------------------------------- */

  protected function propertyToScalarArray($element, $property = 'id') {
    $scalarArray = [];

    foreach($element as $item) {
      $scalarArray[] = $item[$property];
    }

    return $scalarArray;
  }

  protected function propertyToScalar($element, $property = 'id') {
    return isset($element[$property]) ? $element[$property] : $element;
  }

  /* ---------------------------------------------------------------------------------------------
   *                              Default global transformation functions
   * ---------------------------------------------------------------------------------------------- */

  protected function addToParent($data, $field) {
    if (isset($data[$field]) && !empty($data[$field])) {
      foreach($data[$field][0] as $elementKey => &$element) {
        if (!isset($data[$elementKey])) {
          $data[$elementKey] = $element;
          unset($data[$field]);
        } else {
          throw new \Exception('A child and parent field have the same name: "'.$elementKey.'"! Make sure your detail and parent entities don\'t share a field name.');
        }
      }
      return $data;
    } else {
      foreach($data as $elementKey => &$element) {
        if (isset($element[$field])) {
          foreach($element[$field] as $dataKey => $dataValue) {
            if (is_int($dataKey)) {
              foreach($dataValue as $key => $value) {
                if (!isset($element[$key])) {
                  $element[$key] = $value;
                }
                else {
                  throw new \Exception('A child and parent field have the same name: "'.$key.'"! Make sure your detail and parent entities don\'t share a field name.');
                }
              }
            } else {
              if (!isset($element[$dataKey])) {
                $element[$dataKey] = $dataValue;
              }
              else {
                throw new \Exception('A child and parent field have the same name: "'.$dataKey.'"! Make sure your detail and parent entities don\'t share a field name.');
              }
            }
          }

          unset($element[$field]);
        }
      }

      return $data;
    }
  }

  protected function convertArrayToSingleton($data, $field) {
    if (isset($data[$field]) && !empty($data[$field])) {
      $data[$field] = $data[$field][0];
    }

    return $data;
  }

  protected function unsetFields($data, $fields) {
    foreach($fields as $field) {
      foreach($data as $elementKey => &$element) {
        if (is_object($element)) {
          unset($data[$elementKey]);
        } else if (isset($element[$field])) {
          unset($element[$field]);
        }
      }

      if (isset($data[$field])) {
        unset($data[$field]);
      }
    }

    return $data;
  }

  protected function usePropertyValueAsKey($data, $field) {
    foreach($data as $elementKey => &$element) {
      if (isset($element[$field]) && $elementKey !== $element[$field]) {
        $newKey = $element[$field];

        unset($element[$field]);
        $data[$newKey] = $element;
        unset($data[$elementKey]);
      }
    }

    return $data;
  }

  protected function transformFieldKeys($data, $transformation) {
    if (!empty($data)) {
      if (isset($transformation['conversionFormat'])) {
        $conversionParams = [];
        if (isset($transformation['conversionParams'])) {
          $conversionParams = $transformation['conversionParams'];
        }

        $format = $transformation['conversionFormat'];
        if (!isset($data[0]) || !is_array($data[0])) {
          $this->formatKeys($data, $format, $conversionParams);
        } else {
          foreach($data as $key => &$element) {
            $this->formatKeys($element, $format, $conversionParams);
          }
        }
      }
    }

    return $data;
  }

  protected function formatKeys(&$data, $format, $conversionParams) {
    $keys = array_keys($data);

    foreach($keys as $key) {
      $newKey = $this->executeConversion($format, $key, $conversionParams);

      if ($newKey != $key) {
        $data[$newKey] = $data[$key];
        unset($data[$key]);
      }
    }
  }
}
