<?php

namespace Bazookas\CommonBundle\Controller\Base;

use Bazookas\CommonBundle\Entity\Interfaces\EntityInterface;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Form\Form;
use Symfony\Component\HttpFoundation\Request;

abstract class BaseController extends Controller
{
  /**
   * @param Request $request
   * @param Form $form
   * @param EntityInterface $entity
   * @return bool
   */
  protected function handleForm(Request $request, Form &$form, EntityInterface &$entity) {
    $form->handleRequest($request);

    if ($form->isValid() && $form->isSubmitted()) {
      $this->persistEntity($entity);
      return true;
    } else {
      return false;
    }
  }

  /**
   * @param EntityInterface $entity
   */
  protected function persistEntity(EntityInterface $entity) {
    $entityManager = $this->getDoctrine()->getManager();

    if ($entityManager->contains($entity)) {
      $entity->preFormUpdate();
      $entityManager->merge($entity);
    } else {
      $entity->preFormPersist();
      $entityManager->persist($entity);
    }

    $entityManager->flush();
  }

  /**
   * @param string $messageType
   * @param $messageKey
   * @param array $messageParams
   */
  protected function addFlashMessage(string $messageType, $messageKey, $messageParams =  []) {
    $translator = $this->get('translator');
    $this->addFlash(
      $messageType,
      $translator->trans($messageKey, $messageParams)
    );
  }
}
