<?php

namespace Bazookas\CommonBundle\Services\Util;

use Bazookas\CommonBundle\Exception\ConnectionException;

/**
 * Class ConnectorService
 * @package Bazookas\CommonBundle\Services\Util
 * @deprecated To be removed in v2
 */
class ConnectorService
{

  public function getResponse($apiUrl, $method, $apiMethod, $queryArgs = [], $postArgs = [], $postFormat = 'json', $CURLOPTS = [])
  {
    //prepare url
    if (!empty($apiMethod)) {
      $apiUrl = $apiUrl . '/' . $apiMethod;
    }

    $url_param = '?';

    if (!empty($queryArgs)) {
      //need to use &amp; to properly escape some values
      $url_param .= http_build_query($queryArgs, '', '&');
      $apiUrl .= $url_param;
    }

    $data = '';
    if (!empty($postArgs)) {
      switch ($postFormat) {
        case "json":
          $data = json_encode($postArgs);
          break;
        case "xml":
          $data = $postArgs;
          break;
      }
    }

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); //follow redirects
    curl_setopt($ch, CURLOPT_URL, $apiUrl);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

    foreach ($CURLOPTS as $key => $value) {
      curl_setopt($ch, $key, $value);
    }

    // set post data
    $method = strtoupper($method);
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $method);
    if ($method === 'POST' || $method === 'PUT') {
      if (empty($CURLOPTS) || !isset($CURLOPTS[CURLOPT_HTTPHEADER])) {
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json',
            'Content-Length: ' . strlen($data)]
        );
      } else if (isset($CURLOPTS[CURLOPT_HTTPHEADER])) {
        curl_setopt($ch, CURLOPT_HTTPHEADER, $CURLOPTS[CURLOPT_HTTPHEADER]);
      }
      curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
    }
    $rawOutput = curl_exec($ch);

    if ($rawOutput !== false) {
      $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
      $type = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
      $type = explode(';', $type);
      $type = trim($type[0]);
      curl_close($ch);

      $response = null;
      switch ($type) {
        case "text/json":
        case "application/json":
          $response = json_decode($rawOutput, true);
          break;
        case "text/html":
          throw new ConnectionException('Invalid API response format: ' . print_r($rawOutput, true));
          break;
        case "application/xml":
        case "text/xml":
          $response = simplexml_load_string($rawOutput);
          break;
        default:
          $response = $rawOutput;
      }

      return ['status' => $status, 'response' => $response];
    } else {
      throw new ConnectionException('connection erorr: ' . print_r(curl_getinfo($ch), true));
    }
  }

  public function downloadFile($apiUrl, $apiMethod, $savePath, $queryArgs = [], $CURLOPTS = [])
  {
    //prepare url
    if (!empty($apiMethod)) {
      $apiUrl = $apiUrl . '/' . $apiMethod;
    }

    $url_param = '?';

    if (!empty($queryArgs)) {
      //need to use &amp; to properly escape some values
      $url_param .= http_build_query($queryArgs, '', '&');
      $apiUrl .= $url_param;
    }

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true); //follow redirects
    curl_setopt($ch, CURLOPT_URL, $apiUrl);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_BINARYTRANSFER, true);

    foreach ($CURLOPTS as $key => $value) {
      curl_setopt($ch, $key, $value);
    }

    $rawOutput = curl_exec($ch);

    $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    if ($status > 400) {
      curl_close($ch);
      throw new ConnectionException('Error while attempting to download image from \'' . $apiUrl . '\', received a status code: ' . $status);
    }

    curl_close($ch);
    if (file_exists($savePath)) {
      unlink($savePath);
    }
    $fp = fopen($savePath, 'x');
    fwrite($fp, $rawOutput);
    fclose($fp);

    return $savePath;
  }
}
