<?php

namespace Bazookas\CronBundle\Command;

use Bazookas\CronBundle\Entity\Cron;
use Bazookas\CronBundle\Entity\CronHistory;
use Bazookas\CronBundle\Services\Interfaces\CronServiceInterface;
use Doctrine\ORM\EntityManager;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\ArrayInput;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Filesystem\LockHandler;
use Symfony\Component\Process\PhpProcess;
use Symfony\Component\Process\Process;

class RunCronsCommand extends ContainerAwareCommand
{
  private $interval = 5;

  protected function configure()
  {
    $this->setName('crons:run');
  }

  protected function execute(InputInterface $input, OutputInterface $output)
  {
    $entityManager = $this->getContainer()->get('doctrine.orm.entity_manager');
    $consoleDir = 'php '. realpath($this->getContainer()->get('kernel')->getRootDir() . '/../bin/console');
    $iterations = 60 / $this->interval;
    $sleepTime = $this->interval * 1000000;

    for ($i=0; $i < $iterations; $i++) {
      $start = microtime(true);
      //get the crons
      $crons = $this->getCrons($entityManager);

      foreach($crons as $cron) {
        // Create an async process and don't wait for it to complete. Let the crons:run:command handle the errors
        $process = new Process($consoleDir . ' crons:run:command --cron=' . $cron->getId());
        $process->start();
      }

      $timeTaken = floor(max(0, microtime(true) - $start) * 1000000);
      usleep(max(0, $sleepTime - $timeTaken));
    }
  }

  /**
   * @param EntityManager $entityManager
   * @return Cron[]
   */
  protected function getCrons(EntityManager $entityManager) {
    $qb = $entityManager->createQueryBuilder();
    $qb
      ->select('c')
      ->from(Cron::class, 'c')
      ->where($qb->expr()->lt('c.nextRunOn', ':now'))
      ->andWhere($qb->expr()->eq('c.status', ':status1'))
      ->setParameters(array(
        'now' => new \DateTime(),
        'status1' => Cron::STATUS_SCHEDULED,
      ))
    ;

    return $qb->getQuery()->getResult();
  }
}
