<?php

namespace Bazookas\ExportBundle\Command;

use Bazookas\ExportBundle\Services\ExportService;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Filesystem\Filesystem;

class CreateExportCommand extends ContainerAwareCommand {
  private $output;
  private $command = 'export:create';

  protected function configure() {
    $this
      ->setName($this->command)
      ->setDescription('Create an export in the background')
      ->addArgument('configFileName', InputArgument::REQUIRED, 'the name of the temp file that contains the config')
    ;
  }

  protected function execute(InputInterface $input, OutputInterface $output) {
    $this->output = $output;
    $tmpFileName = $input->getArgument('configFileName');

    $handle = fopen($tmpFileName, "r");
    $content = fread($handle, filesize($tmpFileName));
    fclose($handle);

    $configArray = unserialize($content);

    $dataServiceName = $configArray['dataService'];
    if ($dataServiceName == 'defaultService') {
      $dataServiceName = 'bazookas.export.service.DataSelectorService';
    }
    $format = $configArray['format'];
    $fileName = $configArray['fileName'];

    $dataService = $this->getContainer()->get($dataServiceName);
    $exportData = $dataService->getData($configArray);
    $fileCreationService = $this->getContainer()->get('bazookas.export.service.ExportFileCreationService');

    if ($format == ExportService::TYPE_CSV) {
      $response = $fileCreationService->createCsvFile($exportData[0], $fileName . $format);
    }
    if ($format == ExportService::TYPE_XLSX) {
      $response = $fileCreationService->createXlsxFile($exportData, $fileName . $format);
    }

    ob_start();
    $response->sendContent();
    $fileContent = ob_get_contents();
    ob_end_clean();

    $uniqueFileName = $configArray['uniqueFileName'];
    $filePath = 'web/' . $uniqueFileName;
    $fs = new Filesystem();
    $fs->dumpFile($filePath, $fileContent);

    unlink($tmpFileName);
  }
}
