<?php

namespace Bazookas\MediaBundle\Controller\Base;

use Bazookas\AdminBundle\Event\AdminListEvent;
use Bazookas\AdminBundle\Filter\Filter;
use Bazookas\AdminBundle\Util\PageAction;
use Bazookas\AdminBundle\Controller\ListBaseController;
use Bazookas\AdminBundle\Util\ListAction;
use Bazookas\MediaBundle\Exception\LinkedMediaEntityException;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

class MediaBaseController extends ListBaseController
{

  private $addEntityForm;

  function __construct() {
    // make sure parent construct is called!
    parent::__construct();
  }

  protected function checkAccess($_action, $_id) {
    // when returning false on any action, there is no real need to set these
    // these are only usefull for showing/hiding the buttons
    // $this->config->setAccess('add', false);
    // $this->config->setAccess('edit', false);
    // $this->config->setAccess('remove', false);
    return $this->isGranted('ROLE_MEDIA_ADMIN');
  }

  public function listAction(Request $request) {
    //add a default filter
    if (empty($this->filterService->getFilters())) {
      $filter = new Filter();
      $filter->setField('mediaFolder');
      $filter->setOperator('eq');
      $filter->setValue(null);

      $this->filterService->addFilter('mediaFolder', $filter);
    }

    //add an add form
    $entity = new $this->entityMetaData->name();
    $folderId = $this->getFolderIdFromFilters();
    if (!empty($folderId)) {
      $mediaRepo = $this->get('BazookasMediaBundle.repository.BaseMediaRepository');
      $folder = $mediaRepo->getMediaFolderById($folderId);
      $entity->setMediaFolder($folder);
    }
    $this->addEntityForm = $this->createForm($this->config->getAddForm(), $entity);

    $form = $this->processForm($request, $this->addEntityForm, $entity, $this->config->addedMessage);

    if ($form instanceof RedirectResponse) {
      return $form;
    }

    // and then execute the parent list action
    return parent::listAction($request);
  }

  protected function mergeTemplateData(array $templateData, $action) {
    switch($action) {
      case self::ACTION_LIST:
        $mediaRepo = $this->get('BazookasMediaBundle.repository.BaseMediaRepository');
        $folderId = $this->getFolderIdFromFilters();

        $templateData = array_merge($templateData, [
          'folders' => $mediaRepo->getMediaFolders($folderId, $this->isGranted('ROLE_SUPER_ADMIN')),
          'breadcrumb' => $mediaRepo->getFolderBreadCrumb($folderId),
          'currentFolder' => $folderId,
          'folderActions' => $this->getFolderActions(),
          'form' => $this->addEntityForm->createView(),
          'formHeaderLabel' => 'media.entities.' . lcfirst($this->config->getEntityShortClassName()) . '.addForm.header'
        ]);
        break;
    }

    return parent::mergeTemplateData($templateData, $action);
  }

  protected function addListeners($dispatcher)
  {
    $dispatcher->addListener(AdminListEvent::BEFORE_LIST_QUERY, [$this, 'addMediaFoldersToListView']);
  }

  public function addMediaFoldersToListView($event)
  {
    $qb = $event->queryBuilder;
    $folderId = $this->getFolderIdFromFilters();
    if (!empty($folderId)) {
      $qb
        ->andWhere($qb->expr()->eq('e.mediaFolder', ':mediaFolder'))
        ->setParameter('mediaFolder', $folderId)
      ;
    } else {
      $qb->andWhere($qb->expr()->isNull('e.mediaFolder'));
    }
  }

  protected function getFolderIdFromFilters() {
    $folderId = null;
    $filters = $this->filterService->getFilters();
    foreach ($filters as $filter) {
      if ($filter->getField() === 'mediaFolder') {
        $folderId = $filter->getValue();
        break;
      }
    }

    return $folderId;
  }

  public function removeAction(Request $request) {
    //Execute access controle check to make sure the user is allowed to perform the action
    $entity = $this->getRepository()->findOneById($this->id);
    $auth = $this->get('security.authorization_checker');
    if (empty($entity) || !$entity->canDelete($auth, $this->getUser())) {
      return $this->accessDeniedRedirect($request);
    }

    $translator = $this->get('translator');
    try {
      $this->removeEntity($this->id);

      // set flash message
      $this->addFlash(
        'info',
        $translator->trans('admin.list.messages.removed', array(
          '%entity%' => $this->getEntityName()
        ))
      );
    } catch(LinkedMediaEntityException $e) {
      $linkedTo = [];
      foreach($e->getPayload() as $entityName => $items) {
        foreach($items as $item) {
          $message = $translator->trans('media.list.messages.couldNotBeRemoved', array(
            '%entity%' => $this->getEntityName()
          ));
          $linkedTo[$message][] = $item->toHumanReadable();
        }
      }

      $this->addFlash(
        'error', $linkedTo
      );

    }

    $route = $request->get('_route');
    return $this->redirectToRoute($route, array(
      '_locale' => $request->getLocale()
    ));
  }

  /**
   * add an entity using a formbuilder class
   **/
  public function addAction(Request $request) {
    //Execute access controle check to make sure the user is allowed to perform the action
    $entity = new $this->entityMetaData->name();
    $auth = $this->get('security.authorization_checker');
    if (empty($entity) || !$entity->canAdd($auth, $this->getUser())) {
      return $this->accessDeniedRedirect($request);
    }

    $folderId = $this->getFolderIdFromFilters();

    if (!empty($folderId)) {
      $mediaRepo = $this->get('BazookasMediaBundle.repository.BaseMediaRepository');
      $folder = $mediaRepo->getMediaFolderById($folderId);
      $entity->setMediaFolder($folder);
    }

    $form = $this->createForm($this->config->getAddForm(), $entity);
    return $this->handleForm(
      $request,
      $form,
      $entity,
      $this->config->addTemplate,
      $this->config->getTitle($this->action),
      $this->config->addedMessage
    );
  }

  private function getFolderActions() {
    $folderActions = [];

    return $folderActions;

    //TODO implement this

    //TODO refactor this to use the new list actions

    //add the edit button
    $folderEditAction = new ListAction('bazookas_media_media_folder_admin', 'BazookasAdminBundle:List\Partials\ListActions:_editButton.html.twig');
//    $folderEditAction->route = ;
//    $folderEditAction->classes = 'btn-hover-info ti-pencil';
//    $folderEditAction->label = 'admin.actions.edit';
//    $folderEditAction->staticParams = array(
//      '_action' => self::ACTION_EDIT
//    );
//    $folderEditAction->entityParams = array(
//      '_id' => 'id'
//    );

    $folderActions[] = $folderEditAction;

    //add the remove button
    $folderRemoveAction = new ListAction('bazookas_media_media_folder_admin', 'BazookasAdminBundle:List\Partials\ListActions:_removeButton.html.twig');
//    $folderRemoveAction->route = 'bazookas_media_media_folder_admin';
//    $folderRemoveAction->classes = 'btn-hover-danger ti-trash js-confirm-navigation';
//    $folderRemoveAction->label = 'admin.actions.remove';
//    $folderRemoveAction->staticParams = array(
//      '_action' => self::ACTION_REMOVE
//    );
//    $folderRemoveAction->entityParams = array(
//      '_id' => 'id'
//    );

    $folderActions[] = $folderRemoveAction;

    return $folderActions;
  }

  protected function createListActions(Request $request) {
    $route = $request->get('_route');

    if ($this->config->getAccess(self::ACTION_VIEW)) {
      $this->config->addAction('BazookasAdminBundle:List\Partials\ListActions:_viewButton.html.twig', $route);
    }

    if ($this->config->getAccess(self::ACTION_EDIT)) {
      $this->config->addAction('BazookasAdminBundle:List\Partials\ListActions:_editButton.html.twig', $route);
    }

    if ($this->config->getAccess(self::ACTION_REMOVE)) {
      $this->config->addAction('BazookasAdminBundle:List\Partials\ListActions:_removeButton.html.twig', $route);
    }

    if ($this->config->getAccess("add")) {
      $this->config->pageActions[] = new PageAction([
        'path' => '#',
        'label' => 'admin.actions.add',
        'color' => 'success',
        'icon' => 'ti-plus',
        'attr' => [
          'data-target' => '#add-form-panel',
          'data-toggle' => 'collapse',
          'aria-expanded' => false
        ]
      ]);
    }

    //add a create folder button
    $queryParams = array(
      '_action' => 'add',
    );

    $folderId = $this->getFolderIdFromFilters();
    if (!empty($folderId)) {
      $queryParams['folderId'] = $folderId;
    }

    $this->config->pageActions[] = new PageAction([
      'path' => $this->generateUrl('bazookas_media_media_folder_admin', $queryParams),
      'color' => 'info',
      'icon' => 'ti-folder',
      'label' => 'media.folder.addFolder'
    ]);
  }

}
