<?php

namespace Bazookas\MediaBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;
use Symfony\Component\Validator\Constraints as Assert;
use Bazookas\CommonBundle\Entity\Base\BaseEntity;
use Bazookas\MediaBundle\Entity\HiddenMediaFolder;
use Symfony\Component\Security\Core\Authorization\AuthorizationChecker;
use Bazookas\CommonBundle\Entity\Base\BaseCMSUser;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;

/**
* @ORM\Entity
* @ORM\Table(name="media_folder")
* @UniqueEntity(
*     fields={"defaultFolder"},
*     message="media.entities.mediaFolder.fields.defaultFolder.notUnique"
* )
* @ORM\HasLifecycleCallbacks
*/
class MediaFolder extends BaseEntity {

  const TYPE_FOLDER = 'TYPE_FOLDER';
  const TYPE_HIDDEN_FOLDER = 'TYPE_HIDDEN_FOLDER';

  /**
  * @ORM\Column(type="string")
  */
  protected $name;

  /**
  * @ORM\ManyToOne(targetEntity="MediaFolder", inversedBy="children")
  */
  protected $parent;

  /**
  * @ORM\OneToMany(targetEntity="MediaFolder", mappedBy="parent")
  */
  protected $children;

  /**
  * @ORM\Column(type="string")
  */
  protected $type;

  /**
  * @ORM\OneToMany(targetEntity="Bazookas\MediaBundle\Entity\Base\BaseMedia", mappedBy="mediaFolder")
  */
  protected $mediaItems;

  /**
  * @ORM\Column(type="string", nullable=true)
  */
  protected $defaultFolder;

  public function __construct() {
    parent::__construct();

    $this->children = new ArrayCollection();
    $this->mediaItems = new ArrayCollection();
    $this->type = self::TYPE_FOLDER;
  }

  public function getName() {
    return $this->name;
  }

  public function setName($name) {
    $this->name = $name;
    return $this;
  }

  public function getParent() {
    return $this->parent;
  }

  public function setParent($parent) {
    $this->parent = $parent;
    return $this;
  }

  public function getChildren() {
    return $this->children;
  }

  public function setChildren($children) {
    if (is_array($children)) {
      $children = new ArrayCollection($children);
    }

    $this->children = $children;
    return $this;
  }

  public function addChild($child) {
    $this->children->add($child);
    return $this;
  }

  public function removeChild($child) {
    $this->children->removeElement($child);
    return $this;
  }

  public function setType($type) {
    $this->type = $type;
    return $this;
  }

  public function getType() {
    return $this->type;
  }

  public function getMediaItems() {
    return $this->mediaItems;
  }

  public function setMediaItems($items) {
    $this->mediaItems = $items;
    return $this;
  }

  public function addMediaItem($item) {
    $this->mediaItems->add($item);
    return $this;
  }

  public function removeMediaItem($item) {
    $this->mediaItems->removeElement($item);
    return $this;
  }

  public function toHumanReadable($locale = 'nl') {
    return get_class($this)." {name: ".$this->name."}";
  }

  public function getDefaultFolder() {
    return $this->defaultFolder;
  }

  public function setDefaultFolder($id) {
    $this->defaultFolder = $id;
    return $this;
  }

  public function canDelete(AuthorizationChecker $auth, BaseCMSUser $user) {
    return parent::canDelete($auth, $user) && empty($this->defaultFolderType);
  }

  public function getRoleRequiredForView() {
    return 'ROLE_MEDIA_ADMIN';
  }
}
